def omp_parallel_for_firstprivate():
    sum = 0
    i2 = 3

    'omp parallel for reduction(+:sum) firstprivate(i2)'
    for i in range(1,1001):
        sum += i + i2

    known_sum = (1000 * (1000 + 1)) / 2 + i2 * 1000;
    return known_sum == sum



from pythran.tests import TestEnv

import numpy

from tempfile import mkstemp
import unittest
from pythran.typing import List, Set, Dict, NDArray

class TestNormalizeMethods(TestEnv):

    def test_normalize_methods0(self):
        self.run_test("def normalize_methods0(): c = complex(1) ; return complex.conjugate(c)", normalize_methods0=[])

    def test_function_alias0(self):
        self.run_test("""
                      def function_alias0():
                        def p(): return 0
                        g = p
                        return g()""",
                      function_alias0=[])

    def test_function_alias1(self):
        self.run_test("""
                      def function_alias1(n):
                        def p(): return 0
                        def q(): return 1
                        g = p if n else q
                        return g()
                      """,
                      1,
                      function_alias1=[int])

    def test_function_alias2(self):
        self.run_test("""
                      def function_alias2(n):
                        def p(): return 0
                        def q(): return 1
                        if n:
                            g = p
                        else:
                            g = q
                        return g()
                      """,
                      1,
                      function_alias2=[int])

    def test_module_alias0(self):
        self.run_test("def module_alias0(c): import math ; m = math; return m.cos(c)", 1., module_alias0=[float])

    def test_module_alias1(self):
        self.run_test("def module_alias1(c): import math as ma; m = ma; return m.cos(c)", 1., module_alias1=[float])

    def test_module_alias2(self):
        self.run_test("import math as ma\ndef module_alias2(c): m = ma; return m.cos(c)", 1., module_alias2=[float])

    def test_module_alias3(self):
        self.run_test("import math as ma; m = ma\ndef module_alias3(c): return m.cos(c)", 1., module_alias3=[float])

    def test_module_alias4(self):
        self.run_test("""
                      import math as ma
                      def module_alias4(c):
                        import math as ma2
                        m = ma
                        def mab():
                            return m.cos(c) + ma2.cos(c)
                        return mab()""",
                      1.,
                      module_alias4=[float])

    def test_module_alias5(self):
        self.run_test("import math as m2\ndef module_alias5(math): m = m2 ; return m.cos(math)", 1., module_alias5=[float])

    def test_shadow_import0(self):
        self.run_test("def shadow_import0(math): math.add(1)", {1,2}, shadow_import0=[Set[int]])

    def test_shadow_import1(self):
        self.run_test("def shadow_import1(v): math={ v } ; math.add(1)", 1, shadow_import1=[int])

    def test_shadow_import2(self):
        self.run_test("def shadow_import2(s):\n for set in s : set.add(1)", [{1},{2}], shadow_import2=[List[Set[int]]])

    def test_shadow_import3(self):
        self.run_test("def shadow_import3(s): import math ; math = set ; set.add(s, 1)", {1}, shadow_import3=[Set[int]])

    def test_shadow_import4(self):
        self.run_test("import math\ndef shadow_import4(math): math.add(1)", {1}, shadow_import4=[Set[int]])

    def test_builtin_support0(self):
        self.run_test("def builtin_support0(a): import builtins; return builtins.list(a)", [1, 2],  builtin_support0=[List[int]])

    def test_dispatch_clear(self):
        self.run_test("def dispatch_clear(s, d): set.clear(s); dict.clear(d); s.clear(); d.clear() ; return s, d",
                      {1}, {1:1},
                      dispatch_clear=[Set[int],Dict[int,int]])

    def test_dispatch_conjugate(self):
        self.run_test("def dispatch_conjugate(c, n): import numpy; return complex.conjugate(c), numpy.conjugate(n), c.conjugate(), n.conjugate()",
                      2.j, numpy.array([1.j+1.]),
                      dispatch_conjugate=[complex, NDArray[complex, :]])

    def test_dispatch_copy(self):
        self.run_test("def dispatch_copy(d, s, n): import numpy; return dict.copy(d), set.copy(s), numpy.copy(n), d.copy(), s.copy(), n.copy()",
                      {1:1}, {1}, numpy.ones(1),
                      dispatch_copy=[Dict[int,int], Set[int], NDArray[float, :]])

    def test_dispatch_count(self):
        self.run_test("def dispatch_count(s, l): return str.count(s,'1'), list.count(l,1), s.count('1'), l.count(1)",
                      "1", [1],
                      dispatch_count=[str, List[int]])

    def test_dispatch_pop(self):
        self.run_test("def dispatch_pop(l, d): list.pop(l); dict.pop(d,1); l.pop(); d.pop(2); return l, d",
                      [1,2], {1:1, 2:2},
                      dispatch_pop=[List[int], Dict[int,int]])

    def test_dispatch_remove(self):
        self.run_test("def dispatch_remove(s, l): list.remove(l,1); set.remove(s,1); l.remove(2); s.remove(2); return s, l",
                      {1, 2}, [1,2],
                      dispatch_remove=[Set[int], List[int]])

    def test_dispatch_update(self):
        self.run_test("def dispatch_update(s, d): set.update(s, s); dict.update(d,d); s.update(s); d.update(d); return s, d",
                      {1}, {1:1},
                      dispatch_update=[Set[int], Dict[int,int]])

    def test_capture_bound_method(self):
        code = '''
def capture_bound_method(fname, r):
    if r:
        f = open(fname,'w')
        write_line = f.write
    else:
        write_line = fake_write
    for i in range(10):
        write_line(str(i))
    if r:
        f.close()

def fake_write(s):
    return 0'''
        self.run_test(code, "none", False,
                      capture_bound_method=[str, bool])



#from http://rosettacode.org/wiki/Generator/Exponential#Python
#pythran export test(int, int)
#runas test(2, 3)

#FIXME unittest.skip

from itertools import islice, count

def powers(m):
    for n in count():
        yield n ** m

def filtered(s1, s2):
    v, f = next(s1), next(s2)
    while True:
        if v > f:
            f = next(s2)
            continue
        elif v < f:
            yield v
        v = next(s1)

def test(sq, cu):
    squares, cubes = powers(sq), powers(cu)
    f = filtered(squares, cubes)
    return list(islice(f, 20, 30))



""" Module to test slice implementation. """

import numpy
from pythran.typing import List, NDArray

from pythran.tests import TestEnv


class TestSlice(TestEnv):

    """
    Unittest class for code using slices.

    We skip tests for None step as it is equivalent to 1.

    TODO : add tests for 1 == step (None as step)
    """


    def test_empty_slices(self):
        code = 'def empty_slices(x): return x[100:], x[100::2]'
        self.run_test(code, numpy.arange(90),
                      empty_slices=[NDArray[int,:]])


    def test_slice_combination1(self):
        """ Check for "all none" combination. """
        code = """
def slice_combination1(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::-4][begin:end:step],
            a[::4][begin:end:step],
            a[87::-4][begin:end:step],
            a[1::4][begin:end:step],
            a[-3::-4][begin:end:step],
            a[-89::4][begin:end:step],
            a[88:1:-4][begin:end:step],
            a[1:88:4][begin:end:step],
            a[-2:1:-4][begin:end:step],
            a[-89:88:4][begin:end:step],
            a[88:-88:-4][begin:end:step],
            a[2:-1:4][begin:end:step],
            a[-1:-88:-4][begin:end:step],
            a[-88:-1:4][begin:end:step],
            a[:1:-4][begin:end:step],
            a[:87:4][begin:end:step],
            a[:-87:-4][begin:end:step],
            a[:-3:4][begin:end:step])
        """.format(begin=None, end=None, step=None)
        self.run_test(code, numpy.arange(90),
                      slice_combination1=[NDArray[int,:]])

    def test_slice_combination2(self):
        """ Check for positive step combination. """
        code = """
def slice_combination2(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::-4][begin:end:step],
            a[::4][begin:end:step],
            a[87::-4][begin:end:step],
            a[1::4][begin:end:step],
            a[-3::-4][begin:end:step],
            a[-89::4][begin:end:step],
            a[88:1:-4][begin:end:step],
            a[1:88:4][begin:end:step],
            a[-2:1:-4][begin:end:step],
            a[-89:88:4][begin:end:step],
            a[88:-88:-4][begin:end:step],
            a[2:-1:4][begin:end:step],
            a[-1:-88:-4][begin:end:step],
            a[-88:-1:4][begin:end:step],
            a[:1:-4][begin:end:step],
            a[:87:4][begin:end:step],
            a[:-87:-4][begin:end:step],
            a[:-3:4][begin:end:step])
        """.format(begin=None, end=None, step=2)
        self.run_test(code, numpy.arange(90),
                      slice_combination2=[NDArray[int, :]])

    def test_slice_combination3(self):
        """ Check for negative step combination. """
        code = """
def slice_combination3(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step])
# Reversing values with not continuous step is not implemented
#            a[::-4][begin:end:step],
#            a[::4][begin:end:step],
#            a[87::-4][begin:end:step],
#            a[1::4][begin:end:step],
#            a[-3::-4][begin:end:step],
#            a[-89::4][begin:end:step],
#            a[88:1:-4][begin:end:step],
#            a[1:88:4][begin:end:step],
#            a[-2:1:-4][begin:end:step],
#            a[-89:88:4][begin:end:step],
#            a[88:-88:-4][begin:end:step],
#            a[2:-1:4][begin:end:step],
#            a[-1:-88:-4][begin:end:step],
#            a[-88:-1:4][begin:end:step],
#            a[:1:-4][begin:end:step],
#            a[:87:4][begin:end:step],
#            a[:-87:-4][begin:end:step],
#            a[:-3:4][begin:end:step])
        """.format(begin=None, end=None, step=-2)
        self.run_test(code, numpy.arange(90),
                      slice_combination3=[NDArray[int, :]])

    def test_slice_combination4(self):
        """ Check for pos step/no begin/pos end combination. """
        code = """
def slice_combination4(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::4][begin:end:step],
            a[87::-4][begin:end:step],
            a[1::4][begin:end:step],
            a[-3::-4][begin:end:step],
            a[-89::4][begin:end:step],
            a[88:1:-4][begin:end:step],
            a[1:88:4][begin:end:step],
            a[-2:1:-4][begin:end:step],
            a[-89:88:4][begin:end:step],
            a[88:-88:-4][begin:end:step],
            a[2:-1:4][begin:end:step],
            a[-1:-88:-4][begin:end:step],
            a[-88:-1:4][begin:end:step],
            a[:1:-4][begin:end:step],
            a[:87:4][begin:end:step],
            a[:-87:-4][begin:end:step],
            a[:-3:4][begin:end:step])
        """.format(begin=None, end=7, step=2)
        self.run_test(code, numpy.arange(90),
                      slice_combination4=[NDArray[int, :]])

    def test_slice_combination5(self):
        """ Check for pos step/no begin/neg end combination. """
        code = """
def slice_combination5(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step])
# Not implementer for negative end
#            a[::4][begin:end:step],
#            a[87::-4][begin:end:step],
#            a[1::4][begin:end:step],
#            a[-3::-4][begin:end:step],
#            a[-89::4][begin:end:step],
#            a[88:1:-4][begin:end:step],
#            a[1:88:4][begin:end:step],
#            a[-2:1:-4][begin:end:step],
#            a[-89:88:4][begin:end:step],
#            a[88:-88:-4][begin:end:step],
#            a[2:-1:4][begin:end:step],
#            a[-1:-88:-4][begin:end:step],
#            a[-88:-1:4][begin:end:step],
#            a[:1:-4][begin:end:step],
#            a[:87:4][begin:end:step],
#            a[:-87:-4][begin:end:step],
#            a[:-3:4][begin:end:step])
        """.format(begin=None, end=-3, step=2)
        self.run_test(code, numpy.arange(90),
                      slice_combination5=[NDArray[int, :]])

    def test_slice_combination6(self):
        """ Check for pos step/pos begin/no end combination. """
        code = """
def slice_combination6(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::4][begin:end:step],
            a[87::-4][begin:end:step],
            a[1::4][begin:end:step],
            a[-3::-4][begin:end:step],
            a[-89::4][begin:end:step],
            a[88:1:-4][begin:end:step],
            a[1:88:4][begin:end:step],
            a[-2:1:-4][begin:end:step],
            a[-89:88:4][begin:end:step],
            a[88:-88:-4][begin:end:step],
            a[2:-1:4][begin:end:step],
            a[-1:-88:-4][begin:end:step],
            a[-88:-1:4][begin:end:step],
            a[:1:-4][begin:end:step],
            a[:87:4][begin:end:step],
            a[:-87:-4][begin:end:step],
            a[:-3:4][begin:end:step])
        """.format(begin=2, end=None, step=2)
        self.run_test(code, numpy.arange(90),
                      slice_combination6=[NDArray[int, :]])

    def test_slice_combination7(self):
        """ Check for pos step/pos begin/pos end combination. """
        code = """
def slice_combination7(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::4][begin:end:step],
            a[87::-4][begin:end:step],
            a[1::4][begin:end:step],
            a[-3::-4][begin:end:step],
            a[-89::4][begin:end:step],
            a[88:1:-4][begin:end:step],
            a[1:88:4][begin:end:step],
            a[-2:1:-4][begin:end:step],
            a[-89:88:4][begin:end:step],
            a[88:-88:-4][begin:end:step],
            a[2:-1:4][begin:end:step],
            a[-1:-88:-4][begin:end:step],
            a[-88:-1:4][begin:end:step],
            a[:1:-4][begin:end:step],
            a[:87:4][begin:end:step],
            a[:-87:-4][begin:end:step],
            a[:-3:4][begin:end:step])
        """.format(begin=2, end=9, step=2)
        self.run_test(code, numpy.arange(90),
                      slice_combination7=[NDArray[int, :]])

    def test_slice_combination8(self):
        """ Check for pos step/neg begin/no end combination. """
        code = """
def slice_combination8(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step])
# Not implementer for negative begin
#            a[::4][begin:end:step],
#            a[87::-4][begin:end:step],
#            a[1::4][begin:end:step],
#            a[-3::-4][begin:end:step],
#            a[-89::4][begin:end:step],
#            a[88:1:-4][begin:end:step],
#            a[1:88:4][begin:end:step],
#            a[-2:1:-4][begin:end:step],
#            a[-89:88:4][begin:end:step],
#            a[88:-88:-4][begin:end:step],
#            a[2:-1:4][begin:end:step],
#            a[-1:-88:-4][begin:end:step],
#            a[-88:-1:4][begin:end:step],
#            a[:1:-4][begin:end:step],
#            a[:87:4][begin:end:step],
#            a[:-87:-4][begin:end:step],
#            a[:-3:4][begin:end:step])
        """.format(begin=-10, end=None, step=2)
        self.run_test(code, numpy.arange(90),
                      slice_combination8=[NDArray[int, :]])

    def test_step1slice_combination1(self):
        """ Check for "all none" combination. """
        code = """
def step1slice_combination1(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::-1][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=None, end=None, step=None)
        self.run_test(code, numpy.arange(90),
                      step1slice_combination1=[NDArray[int, :]])

    def test_step1slice_combination2(self):
        """ Check for positive step combination. """
        code = """
def step1slice_combination2(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::-1][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=None, end=None, step=2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination2=[NDArray[int, :]])

    def test_step1slice_combination3(self):
        """ Check for negative step combination. """
        code = """
def step1slice_combination3(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::-1][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-2:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=None, end=None, step=-2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination3=[NDArray[int, :]])

    def test_step1slice_combination4(self):
        """ Check for pos step/no begin/pos end combination. """
        code = """
def step1slice_combination4(a):
    begin = {begin}
    end = {end}
    step = {step}
    return(a[::][begin:end:step],
           a[::1][begin:end:step],
           a[87::-1][begin:end:step],
           a[1::1][begin:end:step],
           a[-3::-1][begin:end:step],
           a[-89::1][begin:end:step],
           a[88:1:-1][begin:end:step],
           a[1:88:1][begin:end:step],
           a[-2:1:-1][begin:end:step],
           a[-89:88:1][begin:end:step],
           a[88:-88:-1][begin:end:step],
           a[2:-1:1][begin:end:step],
           a[-1:-88:-1][begin:end:step],
           a[-88:-1:1][begin:end:step],
           a[:1:-1][begin:end:step],
           a[:87:1][begin:end:step],
           a[:-87:-1][begin:end:step],
           a[:-3:1][begin:end:step])
        """.format(begin=None, end=7, step=2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination4=[NDArray[int, :]])

    def test_step1slice_combination5(self):
        """ Check for pos step/no begin/neg end combination. """
        code = """
def step1slice_combination5(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=None, end=-3, step=2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination5=[NDArray[int, :]])

    def test_step1slice_combination6(self):
        """ Check for pos step/pos begin/no end combination. """
        code = """
def step1slice_combination6(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=2, end=None, step=2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination6=[NDArray[int, :]])

    def test_step1slice_combination7(self):
        """ Check for pos step/pos begin/pos end combination. """
        code = """
def step1slice_combination7(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=2, end=9, step=2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination7=[NDArray[int, :]])

    def test_step1slice_combination8(self):
        """ Check for pos step/neg begin/no end combination. """
        code = """
def step1slice_combination8(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=-10, end=None, step=2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination8=[NDArray[int, :]])

    def test_step1slice_combination9(self):
        """ Check for neg step/no begin/pos end combination. """
        code = """
def step1slice_combination9(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=None, end=2, step=-2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination9=[NDArray[int, :]])

    def test_step1slice_combination10(self):
        """ Check for neg step/no begin/neg end combination. """
        code = """
def step1slice_combination10(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=None, end=-10, step=-2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination10=[NDArray[int, :]])

    def test_step1slice_combination11(self):
        """ Check for neg step/pos begin/neg end combination. """
        code = """
def step1slice_combination11(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=85, end=-10, step=-2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination11=[NDArray[int, :]])

    def test_step1slice_combination12(self):
        """ Check for neg step/pos begin/no end combination. """
        code = """
def step1slice_combination12(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=85, end=None, step=-2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination12=[NDArray[int, :]])

    def test_step1slice_combination13(self):
        """ Check for neg step/pos begin/pos end combination. """
        code = """
def step1slice_combination13(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=85, end=3, step=-2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination13=[NDArray[int, :]])

    def test_step1slice_combination14(self):
        """ Check for pos step/neg begin/no end combination. """
        code = """
def step1slice_combination14(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=-3, end=None, step=-2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination14=[NDArray[int, :]])

    def test_step1slice_combination15(self):
        """ Check for neg step/neg begin/pos end combination. """
        code = """
def step1slice_combination15(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=-3, end=4, step=-2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination15=[NDArray[int, :]])

    def test_step1slice_combination16(self):
        """ Check for neg step/neg begin/neg end combination. """
        code = """
def step1slice_combination16(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=-3, end=-10, step=-2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination16=[NDArray[int, :]])

    def test_step1slice_combination17(self):
        """ Check for pos step/pos begin/neg end combination. """
        code = """
def step1slice_combination17(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=3, end=-10, step=2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination17=[NDArray[int, :]])

    def test_step1slice_combination18(self):
        """ Check for pos step/pos begin/neg end combination. """
        code = """
def step1slice_combination18(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=-80, end=80, step=2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination18=[NDArray[int, :]])

    def test_step1slice_combination19(self):
        """ Check for pos step/neg begin/neg end combination. """
        code = """
def step1slice_combination19(a):
    begin = {begin}
    end = {end}
    step = {step}
    return (a[::][begin:end:step],
            a[::1][begin:end:step],
            a[87::-1][begin:end:step],
            a[1::1][begin:end:step],
            a[-3::-1][begin:end:step],
            a[-89::1][begin:end:step],
            a[88:1:-1][begin:end:step],
            a[1:88:1][begin:end:step],
            a[-2:1:-1][begin:end:step],
            a[-89:88:1][begin:end:step],
            a[88:-88:-1][begin:end:step],
            a[2:-1:1][begin:end:step],
            a[-1:-88:-1][begin:end:step],
            a[-88:-1:1][begin:end:step],
            a[:1:-1][begin:end:step],
            a[:87:1][begin:end:step],
            a[:-87:-1][begin:end:step],
            a[:-3:1][begin:end:step])
        """.format(begin=-80, end=-2, step=2)
        self.run_test(code, numpy.arange(90),
        step1slice_combination19=[NDArray[int, :]])

    def test_slice_transpose0(self):
        code = '''
import numpy as np
def slice_transpose0(n):
    base = np.zeros((16, n)).T
    slice1 = base[:10, 10:] # should have shape (10, 6)
    return slice1'''
        self.run_test(code, 16, slice_transpose0=[int])



""" Tests for the time module. """

import pytest
import sys

from pythran.tests import TestEnv


@TestEnv.module
class TestTime(TestEnv):

    """ Tests for the time module. """

    @pytest.mark.skipif(sys.platform == "win32",
                        reason="not supported by winpython for now")
    def test_time_and_sleep(self):
        """ Check time and sleep have the "same" behavior ~0.05 sec. """
        self.run_test("""
            def time_and_sleep():
                import time
                begin = time.time()
                time.sleep(2)
                end = time.time()
                return (end - begin) < 2.05 and (end - begin) > 1.95""",
                      time_and_sleep=[])



import os
import subprocess
from unittest import TestCase

class TestIpythonMagic(TestCase):

    def test_loadext_and_run(self):
        subprocess.check_call(['ipython',
                               os.path.join(os.path.dirname(__file__),
                                            'ipython_script.ipy')])

    def test_loadext_and_run_timeit_twice(self):
        subprocess.check_call(['ipython',
                               os.path.join(os.path.dirname(__file__),
                                            'ipython_script_timeit.ipy')])



#runas BlackScholes(list(range(1,100)), list(range(1,100)), list(range(1,100)), 0.5, 0.76, 12)
#bench BlackScholes(list(range(1,400001)), list(range(1,400001)), list(range(1,400001)), 0.5, 0.76, 400000)
#pythran export BlackScholes(float list, float list, float list, float, float, int)
#pythran export BlackScholes(int list, int list, int list, float, float, int)
import math


def BlackScholes(stock_price, option_strike, option_years, Riskfree,  Volatility, nb_opt):
    RSQRT2PI = 1 / math.sqrt(math.pi * 2)
    A1 = 0.31938153
    A2 = -0.356563782
    A3 = 1.781477937
    A4 = -1.821255978
    A5 = 1.330274429
    call_result = []
    put_result = []
    for opt in range(0, nb_opt) :
        sqrtT = math.sqrt(option_years[opt])
        d1 = math.log(stock_price[opt] / option_strike[opt])
        d1 += (Riskfree + 0.5 * Volatility * Volatility) * option_years[opt]
        d1 /= (Volatility * sqrtT)
        d2 = d1 - Volatility * sqrtT
        K = 1.0 / (1.0 + 0.2316419 * abs(d1))
        CNDD1 = RSQRT2PI * math.exp(-0.5 * d1 * d1) * (K * (A1 + K * (A2 + K * (A3 + K * (A4 + K * A5)))))
        K = 1.0 / (1.0 + 0.2316419 * abs(d2))
        CNDD2 = RSQRT2PI * math.exp(-0.5 * d2 * d2) * (K * (A1 + K * (A2 + K * (A3 + K * (A4 + K * A5)))))
        expRT = math.exp(-Riskfree * option_years[opt])
        call_result.append(stock_price[opt] * CNDD1 - option_strike[opt] * expRT * CNDD2)
        put_result.append(option_strike[opt] * expRT * (1.0 - CNDD2) - stock_price[opt] * (1.0 - CNDD1))
    return call_result, put_result
